<?php

namespace Drupal\entity_browser_table\Plugin\Field\FieldWidget;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeBundleInfo;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\entity_browser\FieldWidgetDisplayInterface;
use Drupal\entity_browser\FieldWidgetDisplayManager;
use Drupal\entity_browser\Plugin\Field\FieldWidget\EntityReferenceBrowserWidget;
use Drupal\taxonomy\Entity\Term;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'entity_reference_browser_table_widget' widget.
 *
 * @FieldWidget(
 *   id = "entity_reference_browser_table_widget",
 *   label = @Translation("Entity Browser - Table"),
 *   multiple_values = TRUE,
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class EntityReferenceBrowserTableWidget extends EntityReferenceBrowserWidget {

  protected $targetId;

  protected $currentLanguage;

  protected $entityBundleInfo;

  protected $entities;

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {

    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.entity_browser.field_widget_display'),
      $container->get('module_handler'),
      $container->get('current_user'),
      $container->get('messenger'),
      $container->get('language_manager'),
      $container->get('entity_type.bundle.info')
    );
  }

  public function __construct(
    string $plugin_id,
    array $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    array $third_party_settings,
    EntityTypeManagerInterface $entity_type_manager,
    FieldWidgetDisplayManager $field_display_manager,
    ModuleHandlerInterface $module_handler,
    AccountInterface $current_user,
    MessengerInterface $messenger,
    LanguageManagerInterface $languageManager,
    EntityTypeBundleInfo $bundleInfo) {
    parent::__construct(
      $plugin_id,
      $plugin_definition,
      $field_definition,
      $settings,
      $third_party_settings,
      $entity_type_manager,
      $field_display_manager,
      $module_handler,
      $current_user,
      $messenger
    );
    $this->currentLanguage = $languageManager->getCurrentLanguage()->getId();
    $this->entityBundleInfo = $bundleInfo;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return parent::defaultSettings();
  }

  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);
    return $form;
  }

  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    $element['#attributes']['class'] = [
      'field--widget-entity-browser-entity-reference',
      'field--widget-entity_reference_browser_table_widget',
    ];
    $element['#attached']['library'][] = 'drp_entity_browser_alter/entity_browser_table';

    return $element;
  }

  /**
   * Builds the render array for displaying the current results as a table.
   *
   * @param string $details_id
   *   The ID for the details element.
   * @param string[] $field_parents
   *   Field parents.
   * @param \Drupal\Core\Entity\ContentEntityInterface[] $entities
   *   Array of referenced entities.
   *
   * @return array
   *   The render array for the current selection.
   */
  protected function displayCurrentSelection($details_id, array $field_parents, array $entities) {
    try {
      $table = [
        '#type'       => 'table',
        '#header'     => $this->buildTableHeaders(),
        '#attributes' => ['class' => ['table--widget-' . $this->getPluginId()]],
        '#empty'      => 'Use the buttons above to add content to this area.',
      ];
      return array_merge($table, $this->buildTableRows($entities, $details_id, $field_parents));
    }
    catch (PluginException $exception) {
      \Drupal::logger('Entity Browser - Table Display')
        ->error(t(
          'Could not get the field widget display: @message',
          ['@message' => $exception->getMessage()]
        ));

      return $table = [
        '#type'   => 'table',
        '#header' => [''],
        '#rows'   => [
          [
            'The field widget could not be found. See the logs for details',
          ],
        ],
      ];
    }

  }

  public function buildTableHeaders() {
    return collect([
      '',
      $this->getFirstColumnHeader(),
      $this->getActionColumnHeader(),
    ])->filter(function ($header) {
      return $header !== NULL;
    })->toArray();
  }

  public function buildTableRows(array $entities, $details_id, $field_parents): array {


    // The "Replace" button will only be shown if this setting is enabled in
    // the widget, and there is only one entity in the current selection.
    $replace_button_access = $this->getSetting('field_widget_replace') && (count($entities) === 1);

    $entities = array_filter($entities);
    $entitiesCollection = [];
    foreach ($entities as $row_id => $entity) {
      if ($entity->hasTranslation($this->currentLanguage) == FALSE) {
        return NULL;
      }
      $entity = $entity->getTranslation($this->currentLanguage);

      $entitiesCollection[] = array_filter([

        'title-preview' => $this->getFirstColumn($entity),
        'actions'       => [
          $this->buildEditButton($entity),
          $this->buildReplaceButton($entity, $entities, $details_id, $row_id, $field_parents),
          $this->buildRemoveButton($entity, $details_id, $row_id, $field_parents),
        ],
        '#attributes'   => [
          'class'          => [
            'item-container',
          ],
          'data-entity-id' => $entity->getEntityTypeId() . ':' . $entity->id(),
        ],
      ]);
    }

    return $entitiesCollection;

  }

  public function buildEditButton(EntityInterface $entity): array {
    return [
      '#type'       => 'submit',
      '#value'      => $this->t('Edit'),
      '#submit'     => ['drp_entity_browser_alter_edit_submit'],
      '#name'       => 'edit-' . $entity->id(),
      '#edit-id'    => $entity->id(),
      '#edit-type'  => $entity->getEntityType(),
      '#attributes' => [
        'class' => ['edit-button'],
      ],
      '#access'     => $this->getEditButtonAccess($entity),
    ];
  }

  public function buildRemoveButton(EntityInterface $entity, string $details_id, $row_id, array $field_parents): array {
    return [
      '#type'                    => 'submit',
      '#value'                   => $this->t('Remove'),
      '#ajax'                    => [
        'callback' => [get_class($this), 'updateWidgetCallback'],
        'wrapper'  => $details_id,
      ],
      '#submit'                  => [
        [
          get_class($this),
          'removeItemSubmit',
        ],
      ],
      '#name'                    => $this->fieldDefinition->getName() . '_remove_' . $entity->id() . '_' . $row_id . '_' . md5(json_encode($field_parents)),
      '#limit_validation_errors' => [array_merge($field_parents, [$this->fieldDefinition->getName()])],
      '#attributes'              => [
        'data-entity-id' => $entity->getEntityTypeId() . ':' . $entity->id(),
        'data-row-id'    => $row_id,
        'class'          => ['remove-button'],
      ],
      '#access'                  => (bool) $this->getSetting('field_widget_remove'),
    ];
  }

  public function buildReplaceButton(EntityInterface $entity, array $entities, string $details_id, $row_id, array $field_parents): array {
    return [
      '#type'                    => 'submit',
      '#value'                   => $this->t('Replace'),
      '#ajax'                    => [
        'callback' => [get_class($this), 'updateWidgetCallback'],
        'wrapper'  => $details_id,
      ],
      '#submit'                  => [
        [
          get_class($this),
          'removeItemSubmit',
        ],
      ],
      '#name'                    => $this->fieldDefinition->getName() . '_replace_' . $entity->id() . '_' . $row_id . '_' . md5(json_encode($field_parents)),
      '#limit_validation_errors' => [array_merge($field_parents, [$this->fieldDefinition->getName()])],
      '#attributes'              => [
        'data-entity-id' => $entity->getEntityTypeId() . ':' . $entity->id(),
        'data-row-id'    => $row_id,
        'class'          => ['replace-button'],
      ],
      '#access'                  => $this->getReplaceButtonAccess($entities),
    ];
  }

  public function getBrands(EntityInterface $entity): string {
    $brands = [];
    if ($entity->hasField('field_brands') || $entity->hasField('field_brand')) {
      $brandIds = ($entity->hasField('field_brands'))
        ? $entity->get('field_brands')->getValue()
        : $entity->get('field_brand')->getValue();
      foreach ($brandIds as $tid) {
        $tid = $tid['target_id'];
        $brands[] = Term::load($tid)->label();
      }
    }
    return (!empty($brands)) ? implode(', ', $brands) : 'No brands set';

  }

  public function getEntityTypeLable(EntityInterface $entity): ?string {
    $bundle = $this->entityTypeManager->getStorage('media_type')
      ->load($entity->bundle());
    return $bundle
      ? $bundle->label()
      : NULL;
  }

  public function getEditButtonAccess(EntityInterface $entity): bool {
    $edit_button_access = $this->getSetting('field_widget_edit') && $entity->access('update', $this->currentUser);
    if ($entity->getEntityTypeId() == 'file') {
      // On file entities, the "edit" button shouldn't be visible unless
      // the module "file_entity" is present, which will allow them to be
      // edited on their own form.
      $edit_button_access &= $this->moduleHandler->moduleExists('file_entity');
    }
    return $edit_button_access;
  }

  public function getReplaceButtonAccess($entities): bool {
    return $this->getSetting('field_widget_replace') && (count($entities) === 1);
  }

  /**
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function getFirstColumn(EntityInterface $entity): array {
    $value = $this->getFieldWidgetDisplay()->view($entity);
    if (is_string($value)) {
      if ($value == ' ') {
        $value = '<i>No title set</i>';
      }
      $value = ['#markup' => $value];
    }
    return $value;
  }

  public function getFirstColumnHeader() {
    return $this->getSetting('field_widget_display') == 'rendered_entity'
      ? 'Thumbnail'
      : 'Title';
  }

  public function getActionColumnHeader() {
    return $this->getSetting('field_widget_edit') || $this->getSetting('field_widget_remove') || $this->getSetting('field_widget_replace')
      ? 'Action'
      : NULL;
  }

  /**
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function getFieldWidgetDisplay(): FieldWidgetDisplayInterface {
    return $this->fieldDisplayManager->createInstance(
      $this->getSetting('field_widget_display'),
      $this->getSetting('field_widget_display_settings') + [
        'entity_type' => $this->fieldDefinition->getFieldStorageDefinition()
          ->getSetting('target_type'),
      ]
    );
  }

}

